
expectancy <- function(p, reward=1, risk=1, numTrades=100) {
  outcomes <- runif(numTrades, min=0, max=1)
  winners <- sum(outcomes <= p)
  losers <- sum(outcomes > p)
  structure(
    (winners*reward - losers*risk)/numTrades,
    data=outcomes,
    p=p,
    class='expectancyAnalysis'
  )
}

expectancyMC <- function(p, reward=1, risk=1, numTrades=100, numSimulations=10000) {
  sapply(
    1:numSimulations, 
    function(x) expectancy(p, reward, risk, numTrades), 
    simplify=FALSE
  )
}


expectationAnalysis <- function(
  experimentName, prWin, numTrades, reward, 
  risk=1, openingBalance=100, simulations=10000, outputPath="~/tmp",
  saveEquityCurves=F
) {
  result <- expectancyMC(prWin, reward, risk, numTrades, simulations)
  
  # Mean expectation from the simulations
  theExps <- sapply(result, function(x) as.numeric(x))
  meanExp <- mean(theExps)
  cat("Mean expectancy is: ", meanExp, "\n")
  
  # Calculate equity curves
  # Assumption that risk express a percentage of capital e.g. risk = 1 implies each trade risk 1% etc.
  equityCurves <- sapply(
    result,
    function(x) {
      outcomes <- attr(x, 'data') <= prWin
      balances <- c(openingBalance)
      rp <- risk/100
      for (idx in 2:length(outcomes)) {
        if ( balances[idx-1] <= 0 ) {
          newBalance <- 0
        } else {
          newBalance <- ifelse(outcomes[idx], balances[idx-1]*(1+rp), balances[idx-1]*(1-rp))
        }
        balances <- c(balances, newBalance)
      }
      balances
    }
  )
  
  
  # Plot expectation results
  png(
    paste(outputPath, paste(experimentName, "_expectancy_hist.png", sep=''), sep='/'), 
    width=1920, height=1080
  )
  hist(theExps, prob=T, xlab=sprintf("Expectancy (p=%3.2f)",prWin),  
       main=sprintf("Experiment %s expectancy distribution (E_{0}=%3.3f, N=%d)", experimentName, meanExp, simulations))
  abline(v=meanExp, lty=1, lwd=5, lend=1, col=rgb(1,0,0,0.5))
  lines(density(theExps), lty=2, lwd=2, col='blue')
  dev.off()
  
  # Plot equity curves
  col.best <- rgb(0, 204/255, 51/255)
  col.average <- rgb(51/255, 153/255, 1)
  col.worst <-rgb(154/255, 0, 51/255)
  png(
    paste(outputPath, paste(experimentName, "_equitycurves.png", sep=''), sep='/'), 
    width=1920, height=1080
  )  
  best <- which(equityCurves[100,] == max(equityCurves[100,]))
  worst <- which(equityCurves[100,] == min(equityCurves[100,]))
  plot(
    x=c(1, numTrades), y=range(c(0, pretty(equityCurves))), type='n', 
    main=paste("Equity curves (Experiment: ", experimentName, ")", sep=""), 
    xlab='Trade', ylab='Equity ($)'
  )
  lower <- apply(equityCurves, 1, min)
  upper <- apply(equityCurves, 1, max)
  polygon(c(1:numTrades, numTrades:1), c(lower, rev(upper)), col=rgb(0.8, 0.8, 0.8, .5), border=NA)
  lines(x=1:numTrades, y=equityCurves[, best], lwd=3, col=col.best)
  lines(x=1:numTrades, y=apply(equityCurves, 1, mean), lwd=2, lty=2, col=col.average)
  lines(x=1:numTrades, y=equityCurves[, worst], lwd=3, col=col.worst)
  legend("topleft", c("Best", "Worst", "Average"), col=c(col.best, col.worst, col.average), bty='n', lty=c(1,1,2), lwd=2)
  dev.off()
  
  # Save equity curves?
  if (saveEquityCurves) {
    write.csv(equityCurves, paste(outputPath, paste(experimentName, "_equitycurves.csv", sep=''), sep='/'))
  }
  
  theExps
}


expecations <- expectationAnalysis(
  experimentName="70perc_1t1", prWin=0.7, numTrades=100, reward=1, 
  risk=1, openingBalance=100, simulations=10000, outputPath="~/dropbox/personal/Dropbox/Public/ff/expectancy",
  saveEquityCurves=T
)
expecations <- expectationAnalysis(
  experimentName="28perc_4t1", prWin=0.28, numTrades=100, reward=4, 
  risk=1, openingBalance=100, simulations=10000, outputPath="~/dropbox/personal/Dropbox/Public/ff/expectancy",
  saveEquityCurves=T
)
